#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Tue Jun 25 18:46:55 2019

@author: salbreg/williamsonj
"""
import csv
import os
import random
import numpy as np
from Run_population_growth import run_population_growth


# =============================================================================
# %%  USEFUL FUNCTIONS AND INFORMATION
# =============================================================================
def isfloat(value):
    "test if value is a float"
    try:
        float(value)
        return True
    except ValueError:
        return False

#obtain the path to where WT movie data is saved
this_directory = os.getcwd()
parent_directory = os.sep.join(this_directory.split(os.sep)[:-1])
# =============================================================================
# %% LOAD DATA ON LIFETIMES AFTER 16hAPF
# TAKE LIFETIMES DETERMINED FROM AVERAGES BETWEEN WILD-TYPES
# =============================================================================

#### define a function which collects data from csv file ####
def load_data_life_times_over_time(file_name):
    "load life time data from csv file"
    life_times_data = []    
    with open(file_name) as csvfile:
        has_header = csv.Sniffer().has_header(csvfile.read(1024))
        csvfile.seek(0)
        readCSV = csv.reader(csvfile, delimiter=',')
        if has_header:
            next(readCSV)  # Skip header row.
        for row in readCSV:
            if isfloat(row[5]):
                if not float(row[5])==0:
                    life_times_data.append([float(row[3]), float(row[5]), float(row[6])])
    return(np.array(life_times_data))
    
#actually load data
file_name = (parent_directory + '/data/lifetimes_fit_allWTs.csv')
data_life_times_over_time=load_data_life_times_over_time(file_name)


    
    
# =============================================================================
# %% RUN SIMULATION AND PLOT THE OUTCOME
# =============================================================================


# Define additional parameters
 
### first phase before SECOND_PHASE_STARTING_TIME ###
STARTING_TIME = 0    
#initial number of cells
INITIAL_NUMBER_CELLS = 18
#mean cell cycle before TRANSIENT_PHASE_STARTING_TIME
MEAN_CELL_CYCLE_BEGINNING = 2.67 
#std of cell cycle before TRANSIENT_PHASE_STARTING_TIME
STD_CELL_CYCLE_BEGINNING = 0.24 * MEAN_CELL_CYCLE_BEGINNING
 #initial mean time to division of  first group of cells
INITIAL_TTD_MEAN = 1.58
#make cells start with nonzero age because the mean first TTD is shorter than lifetime.
INITIAL_AGE = MEAN_CELL_CYCLE_BEGINNING - INITIAL_TTD_MEAN 

### transient phase in between TRANSIENT_PHASE_STARTING_TIME  and SECOND_PHASE_STARTING_TIME
TRANSIENT_PHASE_STARTING_TIME = 3.3 
#end time  of the transient phase
SECOND_PHASE_STARTING_TIME = 14.7
#mean cell cycle time between TRANSIENT_PHASE_STARTING_TIME  and SECOND_PHASE_STARTING_TIME
MEAN_CELL_CYCLE_TRANSIENT = MEAN_CELL_CYCLE_BEGINNING
#std of cell cycle time between TRANSIENT_PHASE_STARTING_TIME  and SECOND_PHASE_STARTING_TIME
STD_CELL_CYCLE_TRANSIENT = 0.32 * MEAN_CELL_CYCLE_TRANSIENT 
#starting time of a pause where cells do not divide
PAUSE_ON = 12.5
#end time of a pause where cells do not divide
PAUSE_OFF = SECOND_PHASE_STARTING_TIME
#starting time of a pause where cells do not age
AGE_ON = 11.2
#end time of a pause where cells do not age
AGE_OFF = 11.2 

#last phase of expansion after SECOND_PHASE_STARTING_TIME

# the mode selection determines how the cell cycles are determined
#in the last phase of expansion
#MODE can be 1 (mean cell cycle time distribution in second phase
#is taken from a fit to experimental data and std is proportional to mean)
#or can be 2 (cell cycle time distribution has a fixed mean and std)
MODE = 1 

COV_CELL_CYCLE = 0.20 
#coefficient of correlation for bivariate normal distribution
RHO_CORRELATION_COEFFICIENT = 0.55
### parameters for apparition of quiescent cells
SWITCH_TIME_ALPHA = 24.7 
HILL_COEFF_ALPHA = 34
SWITCH_TIME_P = 25.7 
HILL_COEFF_P = 34
### parameters for apparition of SOPs
SWITCH_TIME_SOPs_ON = 20.6
SWITCH_TIME_SOPs_OFF = 24
PROBABILITY_SOP = 0.09 

#when MODE=2, these parameters give the mean (MU)
#and std (SIGMA) of the cell cycle time distribution
MU = 4.42
SIGMA = COV_CELL_CYCLE * MU


# =============================================================================
# %% RUN
# =============================================================================

# for random generator initialization
FIRST_SEED = 0
# number of runs of the simulation to execute
N_RUNS = 10
# where to save the data
folder_name_results = 'results_no_transient'
if not os.path.exists(folder_name_results):
    os.makedirs(folder_name_results)
for i in range(0, N_RUNS):
    random.seed(FIRST_SEED + i)
    np.random.seed(FIRST_SEED + i)    
    file_suffix = str(i)    
    THIS_SIMULATION = run_population_growth(file_suffix,
                                            STARTING_TIME,
                                            SECOND_PHASE_STARTING_TIME,
                                            INITIAL_NUMBER_CELLS,
                                            INITIAL_AGE,
                                            MEAN_CELL_CYCLE_BEGINNING,
                                            STD_CELL_CYCLE_BEGINNING,
                                            MEAN_CELL_CYCLE_TRANSIENT,
                                            STD_CELL_CYCLE_TRANSIENT,
                                            SWITCH_TIME_ALPHA,
                                            HILL_COEFF_ALPHA,
                                            SWITCH_TIME_P,
                                            HILL_COEFF_P,
                                            TRANSIENT_PHASE_STARTING_TIME,
                                            SWITCH_TIME_SOPs_ON,
                                            SWITCH_TIME_SOPs_OFF,
                                            PROBABILITY_SOP,
                                            COV_CELL_CYCLE,
                                            data_life_times_over_time,
                                            MU,
                                            SIGMA,
                                            RHO_CORRELATION_COEFFICIENT,
                                            MODE,
                                            PAUSE_ON,
                                            PAUSE_OFF,
                                            AGE_ON,
                                            AGE_OFF,
                                            folder_name_results)
    THIS_SIMULATION.run_population_growth_simulation()
    THIS_SIMULATION.plot_and_save_results()


