#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Tue Jun 25 18:46:55 2019

@author: salbreg/williamsonj
"""
import csv
import os
import random
import numpy as np
from Run_population_growth import run_population_growth

#time resolution
dt = 2.5/60.0
# =============================================================================
# %%  USEFUL FUNCTIONS AND INFORMATION
# =============================================================================
def isfloat(value):
    "test if value is a float"
    try:
        float(value)
        return True
    except ValueError:
        return False

#obtain the path to where WT movie data is saved
this_directory = os.getcwd()
parent_directory = os.sep.join(this_directory.split(os.sep)[:-1])



# =============================================================================
# %% RUN SIMULATION AND PLOT THE OUTCOME
# =============================================================================


# Define additional parameters
 
#simulation starts at 16hAPF
STARTING_TIME = 0.0  

#last phase of expansion after SECOND_PHASE_STARTING_TIME

# the mode selection determines how the cell cycles are determined
#in the last phase of expansion
#MODE can be 1 (mean cell cycle time distribution in second phase
#is taken from a fit to experimental data and std is proportional to mean)
#or can be 2 (cell cycle time distribution has a fixed mean and std)
MODE = 2

COV_CELL_CYCLE = 0.20 
#coefficient of correlation for bivariate normal distribution
RHO_CORRELATION_COEFFICIENT = 0.55



#when MODE=2, these parameters give the mean (MU)
#and std (SIGMA) of the cell cycle time distribution
MU = 4.0 
print('value of mean: '+str(MU))
SIGMA = COV_CELL_CYCLE  * MU
print('value of COV: '+str(SIGMA/MU))
# for random generator initialization
FIRST_SEED = 0
# number of runs of the simulation to execute
N_RUNS = 1
# where to save the data
folder_name_results = 'results'
if not os.path.exists(folder_name_results):
    os.makedirs(folder_name_results)
#initial number of cells
INITIAL_NUMBER_CELLS = 100
INITIAL_AGES = np.zeros( INITIAL_NUMBER_CELLS)
#here I choose an initial condition with a set variation
INITIAL_CELL_CYCLE_TIMES = np.random.normal(MU, SIGMA, INITIAL_NUMBER_CELLS)
for i in range(0, N_RUNS):
    random.seed(FIRST_SEED + i)
    np.random.seed(FIRST_SEED + i)  
    file_suffix = str(i)    
    THIS_SIMULATION = run_population_growth(file_suffix,
                                            STARTING_TIME,
                                            INITIAL_NUMBER_CELLS,
                                            INITIAL_AGES,
                                            MU,
                                            SIGMA,
                                            RHO_CORRELATION_COEFFICIENT,
                                            MODE,
                                            folder_name_results)
    THIS_SIMULATION.INITIAL_CELL_CYCLE_TIMES =  INITIAL_CELL_CYCLE_TIMES 
    THIS_SIMULATION.run_population_growth_simulation()
    THIS_SIMULATION.plot_and_save_results()


